package balance
{
	import flash.display.Bitmap;
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.events.TimerEvent;
	import flash.utils.Timer;	

	public class background extends Sprite
	{
		[Embed(source = "../../lib/background.png")]
		private var bImage:Class;
		
		private var rootHookBounding:Rectangle = new Rectangle(310, 10, 240, 0);
		public const rootX:int = 430;
		public const rootY:int = 10
		
		public static var container:Sprite;				// the container of all display objects
		public static var rootHook:hookShape;			// the first item, it's just a hook
		public static var rootBar:barTool;				// the first toolBar, can't move
		public static var currObj:*;
		
		/* Whenever happens a change in a plumb's
		 * weight, it's initialized with this change.
		 * Used in weight propagation
		 */ 
		public static var runningWeight:Number = 0;
		
		
		public function background():void
		{
			init();
		}
		
		
		private function init():void
		{
			var b:Bitmap = new bImage();	// the background image
			addChild(b);	
			
			container = new Sprite();
			addChild(container);
			
			rootHook = new hookShape();			
			container.addChild(rootHook);
			rootHook.x = rootX;
			rootHook.y = rootY;	
			rootHook.buttonMode = true;
			rootHook.tabEnabled = false;
			rootHook.addEventListener(MouseEvent.MOUSE_DOWN, onRootDown);
			
			rootBar = new barTool();
			rootBar.parentNode = rootHook;
			rootBar.free = false;
			rootBar.attached = true;
			rootBar.buttonMode = false;
			container.addChild(rootBar);
			rootBar.x = rootX;
			rootBar.y = 50;
			
			
			// push left and right collision points
			
			var lP:Point = new Point();
			lP = rootBar.leftPart.localToGlobal(rootBar.leftPart.hookPoint);
			var l:collData = new collData();
			l.collPoint = lP;
			l.free = true;
			l.direction = 0;
			l.objRef = rootBar;
			objArray.collision.push(l);
			
			var rP:Point = new Point();
			rP = rootBar.rightPart.localToGlobal(rootBar.rightPart.hookPoint);
			var r:collData = new collData();
			r.collPoint = rP;
			r.free = true;
			r.direction = 1;
			r.objRef = rootBar;			
			objArray.collision.push(r);				
		}
		
		
		
		private function onRootDown(event:MouseEvent):void
		{
			stage.addEventListener(MouseEvent.MOUSE_UP, onRootUp);
			stage.addEventListener(MouseEvent.MOUSE_MOVE, onRootMove);
			rootHook.startDrag(false, rootHookBounding);
		}		
		
		private function onRootMove(event:MouseEvent):void
		{
			var shift:int = rootX - rootHook.x;
			rootBar.x = rootX - shift;
			
			var lP:Point = new Point();
			lP = rootBar.leftPart.localToGlobal(rootBar.leftPart.hookPoint);
			var l:collData = new collData();
			l.collPoint = lP;
			objArray.collision[0].collPoint = lP;
			
			var rP:Point = new Point();
			rP = rootBar.rightPart.localToGlobal(rootBar.rightPart.hookPoint);
			var r:collData = new collData();
			r.collPoint = rP;
			objArray.collision[1].collPoint = rP;
			
			updatePos(rootBar);
		}
		
		private function onRootUp(event:MouseEvent):void
		{
			stage.removeEventListener(MouseEvent.MOUSE_UP, onRootUp);
			stage.removeEventListener(MouseEvent.MOUSE_MOVE, onRootMove);
			rootHook.stopDrag();
		}
		
		
		
		
		/* Adjusts the lengths of the barTools in the path from obj node
		 * up to the rootBar. If plug is true (happens in barTool attachment),
		 * it stretches the above bars by 25%. If plu is false (happens in
		 * barTool unplug), it shrinks them (in x axis) to their previous
		 * length. It works recursively. It must be followed by updatePos.
		 */ 
		
		public static function adjustLengths(obj:*, plug:Boolean):void
		{
			if (obj.parentNode != rootHook)
			{
				if (obj.parentNode is barTool)
				{
					if (plug)
					{
						obj.parentNode.changeLength(1.25);
					}
					else
					{
						obj.parentNode.changeLength(0.8);
					}
				}
				adjustLengths(obj.parentNode, plug);
			}
		}
		
		
		/* Updates recursively the positions of all objects
		 * in container. From rootBar down to all leafs.
		 * Must be called whenever we attach or unplug an
		 * obj and whenever we change the weight of a plumb.
		 * It calls the followParent method of plumbs and
		 * bars.
		 */
		
		public static function updatePos(obj:*):void
		{
			if (obj.leftChild != null || obj.rightChild !=null)
			{
				if (obj.leftChild != null)
				{
					obj.leftChild.followParent();
					updatePos(obj.leftChild);
				}
				if (obj.rightChild != null)
				{
					obj.rightChild.followParent();
					updatePos(obj.rightChild);
				}
			}
		}
		
		
		/* Propagates the weight change of a plumb. It must
		 * allways be followed by updatePos. The propagation
		 * starts from the obj and, following the path to the
		 * root, it ends at rootBar. It works recursively.
		 * In any step:
		 *** if the parent is a toolBar, it updates the left
		 * and right weights and calls the tollBar's method
		 * change status (changes the slope of bar). It then
		 * transmits the initial weight change to its parent.
		 *** If the parent is a plumb, it updates its left
		 * child weight - the right is allways null - and then
		 * it transmits the initial weight change to its parent.
		 */ 
		
		public static function weightPropagation(obj:*):void
		{
			if (obj.parentNode != rootHook)
			{
				if (obj.parentNode is barTool)
				{
					if (obj.parentNode.leftChild == obj)
					{
						obj.parentNode.leftWeight += runningWeight;
					}
					else
					{
						obj.parentNode.rightWeight += runningWeight;
					}
					obj.parentNode.changeStatus();
				}
				weightPropagation(obj.parentNode);
			}
		}
		
		
		
		
		public static function adoptData(s:String):void
		{
			var tmp:DisplayObject;
			var i:int = 0;
			var sentinel:Boolean = true;
			do
			{
				tmp = container.getChildAt(i);
				if (tmp is varPlumbTool && (tmp as varPlumbTool).symbol == s)
				{
					sentinel = false;
					slider.fromStr.text	= (tmp as varPlumbTool).x_1.toString();
					slider.toStr.text	= (tmp as varPlumbTool).x_2.toString();
					slider.stepStr.text	= (tmp as varPlumbTool).step.toString();
					slider.fromValue 	= (tmp as varPlumbTool).x_1;
					slider.toValue 		= (tmp as varPlumbTool).x_2;
					slider.stepValue 	= (tmp as varPlumbTool).step;
					slider.weightValue	= (tmp as varPlumbTool).weight;
					slider.deModulate();					
				}
				i++;
			}
			while (i < container.numChildren && sentinel);
		}
		
		
		public static function updateSymbols(s:String):void
		{
			var tmp:DisplayObject;
			for (var i:uint = 0; i < container.numChildren ; i++)
			{
				tmp = container.getChildAt(i);
				if (tmp is varPlumbTool && (tmp as varPlumbTool).symbol == s)
				{
					(tmp as varPlumbTool).symbol = slider.nameStr.text;
					(tmp as varPlumbTool).label.text = (tmp as varPlumbTool).symbol;
				}
			}
		}
		
		
		public static function updateX_1s(s:String):void
		{
			var tmp:DisplayObject;
			for (var i:uint = 0; i < container.numChildren ; i++)
			{
				tmp = container.getChildAt(i);
				if (tmp is varPlumbTool && (tmp as varPlumbTool).symbol == s)
				{
					(tmp as varPlumbTool).x_1 = slider.fromValue;
				}
			}
		}	
		
		
		public static function updateX_2s(s:String):void
		{
			var tmp:DisplayObject;
			for (var i:uint = 0; i < container.numChildren ; i++)
			{
				tmp = container.getChildAt(i);
				if (tmp is varPlumbTool && (tmp as varPlumbTool).symbol == s)
				{
					(tmp as varPlumbTool).x_2 = slider.toValue;
				}
			}
		}		
		
		
		public static function updateSteps(s:String):void
		{
			var tmp:DisplayObject;
			for (var i:uint = 0; i < container.numChildren ; i++)
			{
				tmp = container.getChildAt(i);
				if (tmp is varPlumbTool && (tmp as varPlumbTool).symbol == s)
				{
					(tmp as varPlumbTool).step = slider.stepValue;
				}
			}
		}			
			
		
		public static function updateVars(w:Number, s:String):void
		{
			var tmp:DisplayObject;
			for (var i:uint = 0; i < container.numChildren ; i++)
			{
				tmp = container.getChildAt(i);
				if (tmp is varPlumbTool && (tmp as varPlumbTool).symbol == s)
				{
					var dummy:Number = (tmp as varPlumbTool).weight;
					if (dummy > w)
					{
						dummy = - Math.abs(dummy - w);
					}
					else
					{
						dummy = Math.abs(dummy - w);
					}					
					(tmp as varPlumbTool).weight = w;
					(tmp as varPlumbTool).label.text = (tmp as varPlumbTool).symbol;
					(tmp as varPlumbTool).txt.text = (tmp as varPlumbTool).weight.toString();
					
					if ((tmp as varPlumbTool).attached && dummy != 0)
					{
						background.runningWeight = dummy;
						background.weightPropagation(tmp as varPlumbTool);	
						background.updatePos(background.rootBar);
					}
				}
			}
		}
		
		
		
		static public function fadeInOut(obj:DisplayObject, isFadeIn:Boolean):void
		{
			var timer:Timer = new Timer(1, 100);
			timer.addEventListener(TimerEvent.TIMER, TimerTask);
			timer.addEventListener(TimerEvent.TIMER_COMPLETE, TimerCompleteTask);
			timer.start();
			
			function TimerTask(e:TimerEvent):void
			{
				if (obj.alpha < 0.9 && isFadeIn)
				{
					obj.alpha += 0.03;
				}
				if (obj.alpha > 0 && !isFadeIn)
				{
					obj.alpha -= 0.01;
				}
			}
			
			function TimerCompleteTask(e:TimerEvent):void
			{
				timer.stop();
				if (!isFadeIn)
				{
					container.removeChild(obj);
				}
			}
		}		
		
	}
	
}
