package balance
{
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	public class barTool extends balanceNode
	{
		public var mainPart  :barShape;
		public var leftPart  :hookShape;
		public var rightPart :hookShape;
		public var topPart   :linkShape;
		
		public var length:Number;
		public var angle :Number;
		
		public var status:int = 0;		// 0: equilibrium, 1: clockwise right slope, -1: clockwise left slope
		
		private var initPoint:Point = new Point();
		
		public function barTool():void
		{	
			kind   = 0;
			
			length = 1;					// 1 for 150 px length
			angle  = 0;					// 0 degrees for horizontal position			
			
			mainPart = new barShape();
			addChild(mainPart);	
			
			topPart   = new linkShape();
			topPart.x = 0;
			topPart.y = 0;
			addChild(topPart);
			
			leftPart   = new hookShape();
			leftPart.x = -75;
			leftPart.y = 0;
			addChild(leftPart);
			
			rightPart   = new hookShape();
			rightPart.x = 74;
			rightPart.y = 0;
			addChild(rightPart);	
			
			initPoint.x = x;
			initPoint.y = y;
			
			addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
		}
		
		
		public function changeStatus():void
		{
			var oldStatus:int = status;
			leftWeight  = Math.round(leftWeight * 10000) / 10000;
			rightWeight = Math.round(rightWeight * 10000) / 10000;
			var diff:Number = Math.abs(leftWeight - rightWeight);
			if (diff < 0.0009)
			{
				diff = 0;
			}
			if (diff == 0)
			{
				status = 0;
			}
			else 
			{
				if (leftWeight > rightWeight)
				{
					status = -1;
				}
				else
				{
					status = 1;
				}
			}
			if (oldStatus != status)
			{
				angle = status * 15;
				changeAngle(angle);
			}
		}
		
		
		public function changeLength(factor:Number = 1):void
		{
			if (free == false)
			{
				var tmpTurn:Number = -(status * 15);
				changeAngle(tmpTurn);		// turn to horizontal pos
				
				// scale it
				length *= factor;
				mainPart.scaleX  *= factor;
				leftPart.x  = -(length * 150 / 2);
				rightPart.x = (length * 150 / 2) - 1;
				// update collision points
				var i:int = objArray.seekName(name);
				objArray.collision[i].collPoint = leftPart.localToGlobal(leftPart.hookPoint);
				objArray.collision[i + 1].collPoint = rightPart.localToGlobal(rightPart.hookPoint);
				
				tmpTurn = status * 15;
				changeAngle(tmpTurn);		// return to original slope
			}
			
		}
		
		
		public function changeAngle(turn:Number = 0):void
		{
			if (free == false)
			{
				mainPart.rotation = angle;
				var rad:Number = (length * 150) / 2;
				var arc:Number  = turn * Math.PI / 180;
				var newX:Number = Math.cos(arc) * (rad - 1);
				var newY:Number = Math.sin(arc) * (rad - 1);				
				rightPart.x = newX;	
				rightPart.y = newY;
				newX = - Math.cos(arc) * rad;
				newY = - Math.sin(arc) * rad;				
				leftPart.x = newX;	
				leftPart.y = newY;
				//update collition points
				var i:int = objArray.seekName(name);
				objArray.collision[i].collPoint = leftPart.localToGlobal(leftPart.hookPoint);
				objArray.collision[i + 1].collPoint = rightPart.localToGlobal(rightPart.hookPoint);
			}
		}
		
		
		public function followParent():void
		{
			if (parentNode != background.rootHook)
			{
				var i:int = objArray.seekName(parentNode.name);
				var hang:Point = new Point();
				if (parentNode.leftChild == this)
				{
					hang = objArray.collision[i].collPoint;
				}
				else
				{
					hang = objArray.collision[i + 1].collPoint;
				}
				x = hang.x;
				y = hang.y + 21;
				i = objArray.seekName(name);
				objArray.collision[i].collPoint = leftPart.localToGlobal(leftPart.hookPoint);
				objArray.collision[i + 1].collPoint = rightPart.localToGlobal(rightPart.hookPoint);				
			}
		}
		
		
		/* called just before startDrag, used by unPlug
		 */
		public function savePos():void
		{
			initPoint.x = x;
			initPoint.y = y;			
		}
		
		
		public function plug():void
		{
			var test:int = objArray.minDistance(topPart.localToGlobal(topPart.linkPoint));
			if (test > -1)
			{
				objArray.collision[test].free = false;			// this point is no more available
				objArray.collision[test].objRef.free = false;	// parent can't move anymore
				
				// register this as left or right child of parent
				if (objArray.collision[test].direction == 0)
				{
					objArray.collision[test].objRef.leftChild = this;
				}
				else
				{
					objArray.collision[test].objRef.rightChild = this;
				}
					
				// add this to the container
				background.container.addChild(this);
				alpha = 1;
				attached = true;								// stil free
				x = objArray.collision[test].collPoint.x;		// snap to parent's hook
				y = objArray.collision[test].collPoint.y + 21;
				parentNode = objArray.collision[test].objRef;	// register parent as parent node of this
				
				// push entries with free points
				var l:collData = new collData();
				l.collPoint = leftPart.localToGlobal(leftPart.hookPoint);
				l.free = true;
				l.direction = 0;
				l.objRef = this;
				objArray.collision.push(l);
				var r:collData = new collData();
				r.collPoint = rightPart.localToGlobal(rightPart.hookPoint);
				r.free = true;
				r.direction = 1;
				r.objRef = this;
				objArray.collision.push(r);
				
				// update lengths of all barTools upwards
				background.adjustLengths(this, true);
				
				// update positions of all objects
				background.updatePos(background.rootBar);
			}
			else
			{
				x = 45;
				y = 540;
				free = false;
				buttonMode = false;
				background.fadeInOut(this, false);				
			}			
		}
		
		
		public function unPlug():void
		{
			var currentPoint:Point = new Point();
			currentPoint.x = x;
			currentPoint.y = y;
			var dist:Number = Point.distance(initPoint, currentPoint);
			
			if (dist > 30)
			{
				background.adjustLengths(this, false);		// shrink the above lengths
				background.updatePos(background.rootBar);	// update all positions	
				
				// delete the entries in collision list
				i = objArray.seekName(name);
				objArray.collision.splice(i, 2);				
				
				attached = false;
				
				// release the parent's point in collision list
				var i:int = objArray.seekName(parentNode.name);
				if (parentNode.leftChild == this)
				{
					objArray.collision[i].free = true;
					parentNode.leftChild = null;
				}
				else
				{
					objArray.collision[i + 1].free = true;
					parentNode.rightChild = null;
				}
					
				// if parent has no more children then it's free
				if (parentNode != background.rootBar)
				{
					if (parentNode.leftChild == null && parentNode.rightChild == null)
					{
						parentNode.free = true;
					}
				}
				
				x = 45;
				y = 540;
				free = false;
				buttonMode = false;
				background.fadeInOut(this, false);
			}
			else
			{
				x = initPoint.x;
				y = initPoint.y;
			}
		}
		
		
		
		private function onMouseDown(event:MouseEvent):void
		{
			if (free)
			{
				addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
				addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
				savePos();
				startDrag();
			}
		}
		
		private function onMouseMove(event:MouseEvent):void
		{
			if (!attached)
			{
				var test:int = objArray.minDistance(localToGlobal(topPart.linkPoint));
				if (test > -1)
				{
					alpha = 1;
				}
				else 
				{
					alpha = 0.5;
				}
			}
		}		
		
		private function onMouseUp(event:MouseEvent):void
		{
			removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);
			removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			stopDrag();
			if (attached)
			{
				unPlug();
			}
			else
			{
				plug();
			}
		}
	}
	
}